﻿/** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
**/

package player {
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	import flash.display.MovieClip;
	import flash.events.KeyboardEvent;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.external.ExternalInterface;
	import flash.ui.Keyboard;

	public class PlayerHandler {
		private var my_video;
		private var mc_toolbar;
		private var mc_backbutton;
		private var mc_loader;
		private var playPauseHitArea;
		private var exit_button;
		private var toolbarUpper;
		private var scrub;
		private var scrubHitArea;
		private var loaderHitArea;
		private var root;
		private var stage;
		private var loaderMargin:Number;
		private var pressedMenuButtonClass:Class;
		private var doPlayNextVideo:Function;
		private var doBack:Function;
		private var setTimeCurrent:Function;
		private var setTimeDuration:Function;
		private var onPlay:Function;
		private var onPause:Function;
		
		// TODO
		private var isAd = false;
		private var ad = null;
		private var adFMS:String;
		
		private var webTVPlayer:WebTVPlayer;
		private var hideToolbarSoonTimer:Timer;
		private var isToolbarVisible:Boolean;
		private var isDragging:Boolean = false;
		private var isContainer:Boolean;
		private var isHQ:Boolean;
		private var videoFMS:String;
		private var isPlaying:Boolean;
		private var isFirstRun:Boolean;
		private var isStop:Boolean;
		private var videoIndex:Number;
		private var dragStartX:Number;
		private var dragStartY:Number;
		private var dragMayStart:Boolean = false;
		private var dragMoveTreshold:Number = 3;
		private var loaderWidth:Number;
		private var pressedInt;
		private var isSwitching:Boolean = false;
		private var toolbarSlideOut:Boolean;
		private var playPauseButtonAction:Function;
		private var _seekedAlreadyByDragging = false;
		
		private var scrubStartX:Number;
		private var scrubHitAreaStartX:Number;
		private var loadbarStartX:Number;
		
		public function PlayerHandler(my_video, mc_toolbar, mc_backbutton, playPauseHitArea, mc_loader,
									  exit_button, toolbarUpper, scrub, scrubHitArea, loaderHitArea,
									  root, stage,
									  loaderMargin:Number, loaderWidth:Number,
									  doPlayNextVideo:Function, doBack:Function, setTimeCurrent:Function,
									  setTimeDuration:Function, onPlay:Function, onPause:Function,
									  toolbarSlideOut:Boolean, pressedMenuButtonClass:Class) {
			this.my_video = my_video;
			this.mc_toolbar = mc_toolbar;
			this.mc_backbutton = mc_backbutton;
			this.playPauseHitArea = playPauseHitArea;
			this.mc_loader = mc_loader;
			this.exit_button = exit_button;
			this.toolbarUpper = toolbarUpper;
			this.scrub = scrub;
			this.scrubHitArea = scrubHitArea;
			this.loaderHitArea = loaderHitArea;
			
			this.setTimeCurrent = setTimeCurrent;
			this.setTimeDuration = setTimeDuration;
			this.onPlay = onPlay;
			this.onPause = onPause;
			
			this.root = root;
			this.stage = stage;
			this.loaderMargin = loaderMargin;
			this.toolbarSlideOut = toolbarSlideOut;
			this.pressedMenuButtonClass = pressedMenuButtonClass;
			
			this.loaderWidth = loaderWidth;
			this.doPlayNextVideo = doPlayNextVideo;
			this.doBack = doBack;
			
			this.scrubStartX = scrub.x;
			this.scrubHitAreaStartX = scrubHitArea.x;
			this.loadbarStartX = mc_toolbar.loader.loadbar.x;
		}
		
		/**
		 Initialization function. This is called after the script is loaded.
		*/
		public function init(videos:Array, connMode:String) {
			webTVPlayer = new WebTVPlayer(videos);
			my_video.attachNetStream(webTVPlayer.stream);
			
			// Add listeners
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.VIDEO_INITIALIZED, videoInitialized);
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.VIDEO_PLAYHEAD_TIME_CHANGED, videoPlayheadTimeChangedHandler);
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.SEEKING_COMPLETE, seekingCompleteHandler);
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.BUFFER_CHANGED, videoBufferChangedHandler);
			
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.PLAYBACK_STARTED, playbackStarted);
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.PLAYBACK_PAUSED, playbackPaused);
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.PLAYBACK_RESUMED, playbackResumed);
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.PLAYBACK_STOPPED, playbackStopped);
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.PLAYBACK_REPLAYING, playbackReplaying);
			webTVPlayer.addEventListener(WebTVVideoStatusEvent.PLAYBACK_COMPLETED, playbackCompleted);
	
			webTVPlayer.initialize();
	
			mc_toolbar.loader.loadbar.width = 1;
			
			updateConnectionMode(connMode);
	
			// Initialize toolbar hiding
			initializeToolbarHiding();
			hideToolbarSoon();
			
			initializeSeekingByClicking();
			
			scrubHitArea.addEventListener(MouseEvent.MOUSE_DOWN, scrubOnMouseDown);
			
			if (mc_backbutton != null) {
				mc_backbutton.mc_replay.addEventListener(MouseEvent.CLICK,replayClicked);
				mc_backbutton.mc_next.addEventListener(MouseEvent.CLICK,nextClicked);
				mc_backbutton.mc_exit.addEventListener(MouseEvent.CLICK,exitMenuClicked);
			}
			
			ExternalInterface.addCallback("onPlayPause", onPlayPause);
			
			playPauseHitArea.addEventListener(MouseEvent.CLICK,playClicked);
			
			// mc_toolbar.mc_seekforwardbutton.addEventListener(MouseEvent.CLICK,seekForwardClicked);
			// mc_toolbar.mc_seekbackbutton.addEventListener(MouseEvent.CLICK, seekBackwardClicked);
			ExternalInterface.addCallback("seekForward", seekForward);
			exit_button.addEventListener(MouseEvent.CLICK, exitClicked);
			
			if (mc_toolbar.mc_qualitybutton != undefined) {
				mc_toolbar.mc_qualitybutton.button.addEventListener(MouseEvent.CLICK, changeQuality);
			}
			
			root.addEventListener(KeyboardEvent.KEY_DOWN,reportKeyDown);
			
			//Connection mode
			ExternalInterface.addCallback("updateConnectionMode", updateConnectionMode);
		}
		
		function videoInitialized(e:WebTVVideoStatusEvent){
			
			my_video.attachNetStream(webTVPlayer.stream);
			
			initVideo();
			
			if(isAd)
			{
				if (mc_toolbar.mc_qualitybutton != undefined) {
					mc_toolbar.mc_qualitybutton.visible = false;
				}
				
				setQuality();
				
				if(ad.isStreaming)
				{
					// connection_nc.connect(adFMS);
					// connection_nc.client = this;
				}
				else
				{
					playAd();
				}
			}
			playVideo();
		}
		
		function playbackStarted(e:WebTVVideoStatusEvent){
			onPlay.call();
			
			playPauseButtonAction = webTVPlayer.pause;
		}
		
		function playbackPaused(e:WebTVVideoStatusEvent){
			onPause.call();
			
			playPauseButtonAction = webTVPlayer.resume;
		}
		
		function playbackResumed(e:WebTVVideoStatusEvent){
			onPlay.call();
			
			playPauseButtonAction = webTVPlayer.pause;
		}
		
		function playbackStopped(e:WebTVVideoStatusEvent){
			onPause.call();
			
			playPauseButtonAction = webTVPlayer.play;
		}
		
		function playbackReplaying(e:WebTVVideoStatusEvent){
			if (mc_backbutton != null) {
				mc_backbutton.visible = false;
			}
			
			onPlay.call();
			
			playPauseButtonAction = webTVPlayer.pause;
		}
		
		function playbackCompleted(e:WebTVVideoStatusEvent){
			if (mc_backbutton != null) {
				mc_backbutton.visible = true;
			}
			
			onPause.call();
			
			playPauseButtonAction = webTVPlayer.replay;
		}
		
		function initializeSeekingByClicking():void{
			loaderHitArea.addEventListener(MouseEvent.CLICK, onLoaderClick);
		}
		
		function onLoaderClick(e:MouseEvent):void{
			if(_seekedAlreadyByDragging){
				// Already seeked
				// Notice! Scrub mouse up event comes before this
			} else {
				seekVideoToScrubPosition(mc_toolbar.loader.mouseX);
			}
			_seekedAlreadyByDragging = false;
		}

		public function seekVideoBy(by:Number):void {
			if(isAd || webTVPlayer.duration < 0){
				return;
			}
			
			var seekTo:Number = webTVPlayer.playheadTime + by;
			
			if (seekTo < 0) {
				seekTo = 0;
			} else if (seekTo > webTVPlayer.duration) {
				seekTo = webTVPlayer.duration;
			}
			
			webTVPlayer.seekTo(seekTo);
		}
		
		function initializeToolbarHiding():void {
			hideToolbarSoonTimer = new Timer(4000, 1);
			hideToolbarSoonTimer.addEventListener(TimerEvent.TIMER, function(e:Event):void{
				if(this.keepToolbarVisible){
					hideToolbarSoon();
				} else {
					hideToolbarNow();
				}
			});
			
			stage.addEventListener(MouseEvent.CLICK, function(e:Event):void{
				if(!isToolbarVisible){
					showToolbar();
					hideToolbarSoon();
				}
			});
		}
		
		function videoPlayheadTimeChangedHandler(e:WebTVVideoStatusEvent){
			var playheadTime:Number = e.playheadTime;
			
			if(!isDragging){
				// Don't update scrub position if user is dragging at the moment
				trace("----------------------- UPDATING SCRUP POSITION TO: " + e.playheadTime);
				updateScrubPosition(e.playheadTime);
			}
			
			if(isLoaderAnimationVisible() && playheadTime > 0.1){
				// Hide loader as soon as video starts playing
				hideLoaderAnimation();
			}
			
			setCurrentTime(playheadTime, webTVPlayer.duration);
		}
		
		function setQuality()
		{
			if (mc_toolbar.mc_qualitybutton == undefined) {
				return;
			}
			
			mc_toolbar.mc_qualitybutton.label.mouseEnabled = false;
			
			if(/* video.length <= 1 || */ isAd)
			{
				mc_toolbar.mc_qualitybutton.visible = false;
				return;
			}
			else
			{
				if(!isContainer)
				{
					var pos = 0;
					mc_toolbar.mc_qualitybutton.label.text = "" + webTVPlayer.firstVideo.bitrate;
					isContainer = true;
					
					if(false /*video.length > 1 */)
					{
						for(var i = 0; i < video.length; i++)
						{
							
							var button:QualityClip = new QualityClip();
							
							//trace(button);
							button.label.text = video[i].bitrate;
							button.name = video[i].bitrate;
							button.Tag = video[i].bitrate;
							button.label.mouseEnabled = false;
							qualityContainer.addChild(button);
							button.addEventListener(MouseEvent.CLICK,qualityButton);
							button.y = pos;
							
							pos += 47;
						}
						
						this.addChild(qualityContainer);
						qualityContainer.x = 473;
						var yPos = 297.3 - (video.length * 47);
						qualityContainer.y = yPos-1;
						
					}
				}
				
				
				mc_toolbar.mc_qualitybutton.buttonMode = true;
				mc_toolbar.mc_qualitybutton.gotoAndStop("HQ");
				mc_toolbar.mc_qualitybutton.visible = true;
				isHQ = false;
			}
		}
		
		/*Video data*/
		
		function initVideo()
		{
			/*
			if(webTVPlayer.isStreaming)
			{
				videoFMS = webTVVideo.link.substr(0,webTVVideo.link.lastIndexOf("/")+1);
				
				trace("FMS " + videoFMS);
				
				for(var i = 0; i < video.length; i++)
				{
					
					video[i].link = video[i].link.substr(video[i].link.lastIndexOf("/") + 1, video[i].link.length);
					var type = video[i].link.substr(video[i].link.lastIndexOf(".") + 1, video[i].link.length);
					video[i].link = type + ":" + video[i].link.substr(0,video[i].link.lastIndexOf("."));
				}
			}
			else
			{
				
			}
			*/
			
			initAd();
			// setQuality();
		}
		
		function initAd()
		{
			if(ad != null)
			{
				var inParam = ad.video.link;
				trace("initAd() " + inParam);
				if(inParam.indexOf("rtmp") >= 0) 
				{
					adFMS = inParam.substr(0,inParam.lastIndexOf("/")+1);
					trace("adFMS " + adFMS);
					
					inParam = inParam.substr(inParam.lastIndexOf("/") + 1, inParam.length);
					var type = inParam.substr(inParam.lastIndexOf(".") + 1, inParam.length);
					ad.video.link = type + ":" + inParam.substr(0,inParam.lastIndexOf("."));
					ad.isStreaming = true;
				}
				else
				{
					ad.isStreaming = false;
				}
				
				isAd = true;
				
			}
		}
		
		function setStreaming()
		{
			throw new Error("Unimplemented");
			/*
			stream_ns = new DynamicStream(connection_nc);        //plus other stuff if you need.
			loaded=stream_ns.bytesLoaded;
			total=stream_ns.bytesTotal;
			stream_ns.bufferTime = 5; 
			my_video.attachNetStream(stream_ns);
			stream_ns.addEventListener(NetStatusEvent.NET_STATUS, onStatusEvent);
			stream_ns.client = meta;
			// stream_ns.manualSwitchMode(isDynamic);
			*/
					
		}
		
		function onAdStop()
		{
			// flash.utils.clearInterval(time_interval);
		
			mc_toolbar.loader.loadbar.width = 1;	
			mc_loader.visible = true;
			mc_loader.x = 0;
			mc_loader.y = 0;
			
			
			trace("onAdStop");
			
			isAd = false;
			if(webTVPlayer.isStreaming)
			{
				trace("onAdStop " + webTVPlayer.isStreaming);
				// connection_nc.connect(videoFMS);
				// connection_nc.client = this;
			}
			else
			{
				trace("onAdStop progressive");
				playVideo();
			}
		}
		
		
		function playAd()
		{
			mc_toolbar.loader.loadbar.width = 1;
			if(ad.isStreaming)
			{
				setStreaming();
				var dsi:DynamicStreamItem = new DynamicStreamItem();
				dsi.addStream( ad.video.link, 200 );
				trace("playAd streaming " + ad.video.link); 
				// stream_ns.startBufferLength = 5;
				// stream_ns.preferredBufferLength = 5;
				// stream_ns.startPlay( dsi );
			}
			else
			{
				// setProgressive();
				webTVPlayer.stream.play(ad.video.link);
			}
			
			onPlay.call();
			
			isPlaying = true;
			isFirstRun = true;
			
			// time_interval  = flash.utils.setInterval(checkTime, 800, webTVPlayer.stream, this);
		}
		
		function playVideo() {
			
			if (mc_backbutton != null) {
				mc_backbutton.visible = false;
			}
			
			isStop = false;
			
			webTVPlayer.play();
			setQuality();
			
			onPlay.call();
			
			isPlaying = true;
			isFirstRun = true;
		}
		
		function onStop() {
			throw new Error("Unimplemented onStop");
			/*
			if(!isStop) {
				
				trace("onStop");
				webTVPlayer.pause();
		
				isStop = true;
				isPlaying = false;
				isFirstRun = true;
				
				videoIndex += 1;
				
				if(videoCount != 3) {
					mc_backbutton.mc_next.disable();
				}
				
				mc_backbutton.visible = true;
			}
			*/
		}
		
		
		function clearStop() {
			if(isStop) {
				isStop = false;
				isPlaying = true;
				videoIndex -= 1;
				if(videoIndex < 0) {
					videoIndex = 0;
				}
				
				if (mc_backbutton != null) {
					mc_backbutton.visible = false;
				}
			}
		}
		
		function videoBufferChangedHandler(e:WebTVVideoStatusEvent){
			var currentBufferWidth:Number = (e.bytesLoaded / e.wholeVideoBytesTotal) * loaderWidth;
			
			var element = mc_toolbar.loader.loadbar;
			
			element.width = currentBufferWidth;
			element.x = loadbarStartX + (e.serverSeekBytesStart / e.wholeVideoBytesTotal) * loaderWidth;
			
			trace("element.width: " + element.width + ", " + loaderWidth + ", x: " + element.x +
				  " bytes total: " + e.bytesLoaded + " whole bytes: " + e.wholeVideoBytesTotal);
		}
		
		/**
		 Updates the scrub position regarding the video playback
		 This method replaces old function "setScrubPos"
		 The actual scrub position is set using function setScrubPosition
		*/
		function updateScrubPosition(time)
		{
			setScrubPosition(videoTimeToScrubPosition(time, webTVPlayer.duration));
			
			// Force y to 0, always!
			//mc_toolbar.loader.scrub.y = 0;
		}
		
		function videoTimeToScrubPosition(time:Number, duration:Number){
			return (loaderMargin / 2) + ((time / duration) * loaderWidth);
		}
		
		function scrubPositionToVideoTime(position:Number, duration:Number){
			return ((position - (loaderMargin / 2)) / loaderWidth) * duration;
		}
		
		/**
		 @param position Scrub position, this can be for example mouseX of the toolbar
		*/
		function setScrubPosition(position:Number)
		{
			var adjustedPosition = adjustScrubPosition(position);
			
			scrub.x = scrubStartX + adjustedPosition;
			//mc_toolbar.loader.scrub.y = 0;
			
			scrubHitArea.x = scrubHitAreaStartX + adjustedPosition;
			//mc_toolbar.loader.scrubHitArea.y = -8.80;
		}
		
		/**
		 Seeks video to scrub position
		 */
		function seekVideoToScrubPosition(scrubPosition)
		{
			scrubPosition = adjustScrubPosition(scrubPosition);
			var videoTime = this.scrubPositionToVideoTime(scrubPosition, webTVPlayer.duration);
			
			if(isAd || webTVPlayer.duration < 0){
				return;
			}
			
			webTVPlayer.seekTo(videoTime);
		}
		
		function seekingCompleteHandler(e:WebTVVideoStatusEvent) {
			
			// Hide toolbar rather quickly after seek for the 
			// sake of performance
			// var hideToolbarAfterSeekTimer:Timer = new Timer(500, 1);
			// hideToolbarAfterSeekTimer.addEventListener(TimerEvent.TIMER, hideToolbar);
			// hideToolbarAfterSeekTimer.start();
			
			isDragging = false;
			keepToolbarVisible = false;
			
			trace("------------------ SEEKING COMPLETE TO: " + e.playheadTime);
			updateScrubPosition(e.playheadTime);
		}
		
		function scrubOnMouseDown(event:MouseEvent)
		{
			dragMayStart = true;
			dragStartX = event.stageX;
			dragStartY = event.stageY;
			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, scrubDragMove);
			stage.addEventListener(MouseEvent.MOUSE_UP, scrubOnMouseUp);
			
			debugDragInfo("scrubOnMouseDown", event.stageX, event.stageY);
		}
		
		function scrubDragMove(event:MouseEvent)
		{
			if(isDragging){
				setScrubPosition(mc_toolbar.loader.mouseX);
			} else {
				if(dragMayStart && Math.abs(event.stageX - dragStartX) >= dragMoveTreshold){
					// Drag starting!
					setScrubPosition(mc_toolbar.loader.mouseX);
					isDragging = true;
					keepToolbarVisible = true;
					trace("Drag started");
				}
			}
		}
		
		function scrubOnMouseUp(event:MouseEvent)
		{
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, scrubDragMove);
			stage.removeEventListener(MouseEvent.MOUSE_UP, scrubOnMouseUp);
			
			if(isDragging){
				dragMayStart = false;
				seekVideoToScrubPosition(mc_toolbar.loader.mouseX);
			} else {
				// Restart the scrub position updating
				// The same thing is done after seeking is complete
				trace("Adding MOUSECLICK");
			}
			
			_seekedAlreadyByDragging = true;
			
			debugDragInfo("scrubOnMouseUp", event.stageX, event.stageY);
		}
		
		function debugDragInfo(message:String, mouseX:Number, mouseY:Number){
			var debugDrag:Boolean = true;
			if(debugDrag){
				trace("dragMayStart: " + dragMayStart + ", isDragging: " + isDragging + ", dragStartX: " + dragStartX + ", dragStartY: " + dragStartY + ", x: " + mouseX + ", y: " + mouseY);
			}
		}
		
		/**
		 Adjusts the given position
		 
		 Ensures that the position is
		 - Above 0 point (which is 0 minus scrub width / 2)
		 - Below max point (which is width + scrub width / 2)
		 - Below the buffered video point if seeking outside the buffered area is not allowed
		 
		 @params Number position
		 @returns Number adjusted scrub position
		*/
		 
		function adjustScrubPosition(position){
				// Upper limit is the loader full width or the buffered position
			var loaderWidth:Number = loaderWidth + (loaderMargin / 2);
			
			trace("END TIME: " + webTVPlayer.bufferEndTime);
			
			var bufferedVideoWidth:Number = this.videoTimeToScrubPosition(webTVPlayer.bufferEndTime, webTVPlayer.duration);
			
			
			var lowerLimit:Number = loaderMargin / 2;
			var upperLimit:Number;
			
			if(webTVPlayer.serverSeekAvailable){
				upperLimit = loaderWidth;
			} else {
				upperLimit = Math.min(loaderWidth, bufferedVideoWidth);
			}
			
			trace("UPPER LIMIT: " + upperLimit);
			
			if(position < lowerLimit){
				position = lowerLimit;
			}
			if(position > upperLimit){
				position = upperLimit;
			}
			
			return position;
		}
		
		function isLoaderAnimationVisible():Boolean{
			return this.mc_loader.visible == true;
		}
		
		function hideLoaderAnimation():void{
			this.mc_loader.visible = false;
			
			if (this.mc_loader.stopProgress != undefined) {
				this.mc_loader.stopProgress();
			}
		}
		
		function setCurrentTime(time:Number, duration:Number, ignoreIfInvisible:Boolean = true)
		{
			var streamSecondsPast:Number = time;
			var streamSecondsLeft:Number = duration - streamSecondsPast;
			
			if(mc_toolbar.visible == false && ignoreIfInvisible)
			{
				trace("Not updating time values");
			}
			else
			{
				var minutesCurrent:Number = Math.floor(time/60);
				var secondsCurrent = Math.floor(time%60);
				if (secondsCurrent<10) {
					secondsCurrent = "0"+secondsCurrent;
				}
				
				var minutesDuration:Number = Math.floor(duration/60);
				var secondsDuration = Math.floor(duration%60);
				if (secondsDuration<10) {
					secondsDuration = "0"+secondsDuration;
				}
				
				var timeDuration:String = minutesDuration + ":" + secondsDuration;
				var timeCurrent:String = minutesCurrent + ":" + secondsCurrent;
				
				setTimeCurrent.call(null, timeCurrent);
				setTimeDuration.call(null, timeDuration);
			}
		}
		
		function hideToolbarSoon():void{
			hideToolbarSoonTimer.reset();
			hideToolbarSoonTimer.start();
		}
		
		function hideToolbarNow():void{
			mc_toolbar.visible = false;
			toolbarUpper.visible = false;
			isToolbarVisible = false;
		}
		
		function showToolbar():void{
			mc_toolbar.visible = true;
			toolbarUpper.visible = true;
			isToolbarVisible = true;
		}
		
		function set keepToolbarVisible(value:Boolean):void{
			if(value == true){
				hideToolbarSoonTimer.stop();
			} else {
				hideToolbarSoon();
			}
		}
		
		function removePressed(clip:MovieClip, mc:MovieClip)
		{
			flash.utils.clearInterval(clip.pressedInt);
			
			mc.removeChild(mc.getChildByName("pressed"));
		}
		
		
		public function createPressedMenuButton() {
			if (pressedMenuButtonClass == null) {
				return null;
			}
			
			return new pressedMenuButtonClass();
		}
		
		function replayClicked(evt:MouseEvent){
				
			var pressed = createPressedMenuButton();
			
			if (pressed != null) {
				mc_backbutton.mc_replay.container_mc.addChild(pressed);
				pressed.name = "pressed";
				pressedInt = flash.utils.setInterval(removePressed, 100, this, mc_backbutton.mc_replay.container_mc);
			}
			
			trace("ExternalInterface.call(\"startvibra\");")
			ExternalInterface.call("startvibra");
			clearStop();
			onPlay.call();
			webTVPlayer.seekTo(0);
		}
		
		function nextClicked(evt:MouseEvent){
			/*
			if(videoCount != 3) 
			{
				return;
			}
			*/
			var pressed = createPressedMenuButton();
			
			if (pressed != null) {
				mc_backbutton.mc_next.addChild(pressed);
				pressed.name = "pressed";
				pressedInt = flash.utils.setInterval(removePressed, 100, this, mc_backbutton.mc_next);
			}
			
			doPlayNextVideo.call();
		}
		
		function exitMenuClicked(evt:MouseEvent){
			
			var pressed = createPressedMenuButton();
			
			if (pressed != null) {
				mc_backbutton.mc_exit.container_mc.addChild(pressed);
				pressed.name = "pressed";
				pressedInt = flash.utils.setInterval(removePressed, 100, this, mc_backbutton.mc_exit.container_mc);
			}
			
			trace("ExternalInterface.call(\"startvibra\");")
			ExternalInterface.call("startvibra");
			
			doBack();
		}
		
		function playClicked(evt:MouseEvent){
			//var playButton:MovieClip = mc_toolbar.mc_playbutton;
			//addPressEffect(playButton.playPauseHitArea);
			
			playPauseButtonAction();
		}
		
		function onPlayPause()
		{
			playOrPause(isPlaying);
		}
		
		public function playOrPause(pausePlayback:Boolean)
		{
			var playButton = mc_toolbar.mc_playbutton;
			
			if (((playButton as MovieClip) != null) && (playButton.playPauseHitArea != undefined)) {
				this.addPressEffect(playButton.playPauseHitArea);
			}
			
			if (mc_backbutton != null) {
				mc_backbutton.visible = false;
			}
			
			if(pausePlayback) {
				trace("movePlay");
				webTVPlayer.stream.pause();
				onPause.call();
				isPlaying = false;
				keepToolbarVisible = true;
			}
			else {
				trace("movePause");
				webTVPlayer.stream.resume();
				onPlay.call();
				isPlaying = true;
				keepToolbarVisible = false;
			}
			
			trace("ExternalInterface.call(\"startvibra\");")
			ExternalInterface.call("startvibra");
			isStop = false;
		}
		
		function addPressEffect(hitArea:MovieClip){
			
			trace("Adding press effect");
			
			var pressed = createPressedMenuButton();
			
			if (pressed == null) {
				return;
			}
			
			pressed.width = hitArea.width;
			pressed.height = hitArea.height;
			hitArea.addChild(pressed);
			
			var removePressEffectTimer:Timer = new Timer(100, 1);
			removePressEffectTimer.addEventListener(TimerEvent.TIMER, function(e:Event){
				trace("Removing press effect");
				hitArea.removeChild(pressed);
			});
			removePressEffectTimer.start();
		}
		
		function seekForwardClicked(evt:MouseEvent){
			
			if(isAd)
			{
				return;
			}
		
			trace("ExternalInterface.call(\"startvibra\");")
			ExternalInterface.call("startvibra");
			seekForward();
		}
		
		function seekForward() {
			
			if(!isPlaying)
			{
				return;
			}
			
			var pressed = createPressedMenuButton();
			
			if (pressed != null) {
				mc_toolbar.mc_seekforwardbutton.container_mc.addChild(pressed);
				pressed.name = "pressed";
				pressedInt = flash.utils.setInterval(removePressed, 100, this, mc_toolbar.mc_seekforwardbutton.container_mc);
			}
			
			if(!isPlaying) {
				trace("not playing");
				return;
			}
			
			var temp = webTVPlayer.stream.time + 30;
			
			var factor = temp / webTVPlayer.duration * 208.9;
			trace("Seek pos " + factor);
			
			if(temp < webTVPlayer.duration) {
				webTVPlayer.stream.seek(temp);
			}
			else {
				webTVPlayer.stream.seek(webTVPlayer.duration - 10);
			}
		}
		
		function seekBackwardClicked(evt:MouseEvent){
			
			if(isAd)
			{
				return;
			}
			
			trace("ExternalInterface.call(\"startvibra\");")
			ExternalInterface.call("startvibra");
			seekBackward();
		}
		
		function seekBackward() {
			
			if(!isPlaying) {
				return;
			}
			
			if(!isStop) {
				clearStop();
			}
			
			var pressed = createPressedMenuButton();
			
			if (pressed != null) {
				mc_toolbar.mc_seekbackbutton.container_mc.addChild(pressed);
				pressed.name = "pressed";
				pressedInt = flash.utils.setInterval(removePressed, 100, this, mc_toolbar.mc_seekbackbutton.container_mc);
			}
			
			trace("Seek back Time: " + webTVPlayer.stream.time + "Duration " + webTVPlayer.duration);
			var temp = webTVPlayer.stream.time - 30;
			
			var factor = temp / webTVPlayer.duration * 208.9;
			
			/*
			if(!canSeekTo(factor) && !isStreaming)
			{
				trace("Can seek forward - progressive");
				return;
			}
			*/
			
			if(temp >= 0){
				trace("Seek to time" + temp);
				webTVPlayer.stream.seek(temp);
			}
			else {
				webTVPlayer.stream.seek(5);
			}
		}
		
		function exitClicked(e:MouseEvent){
			trace("function exitClicked");
			var hitArea:MovieClip = toolbarUpper.exitButtonHitArea;
			addPressEffect(hitArea);
			
			trace("ExternalInterface.call(\"startvibra\");")
			ExternalInterface.call("startvibra");
			
			webTVPlayer.stop();
			doBack();
		}
		
		function changeQuality(evt:MouseEvent)
		{
			trace("changeQuality");
			if(isSwitching)
			{
				return;
			}
			
			isSwitching = false;
			
			/*
			if(qualityContainer.visible)
			{
				trace("showHide");
				qualityContainer.visible = false;
			}
			else
			{
				qualityContainer.visible = true;
			}
			*/
		}
		
		function reportKeyDown(event:KeyboardEvent):void
		{
			trace("Key Pressed: " + event.keyCode);
			var code = event.keyCode;
			
			if (code == Keyboard.RIGHT)
			{
				// seekForward();
				trace("Seek forward");
				
			} 
			else if (code == Keyboard.LEFT)
			{
				// seekBackward();
				trace("Seek backwards");
			}
			else if (code == Keyboard.UP)
			{
				// soundUp();
				trace("Seek backwards");
			}
			else if (code == Keyboard.DOWN)
			{
				// soundDown();
				trace("Seek backwards");
			}
			else if (code == Keyboard.ENTER)
			{
				onPlayPause();
				trace("Seek backwards");
			}
			
			else if (code == Keyboard.SPACE)
			{
				onPlayPause();
				trace("Seek backwards");
			}
			
			else {
				trace("You have pressed: "+ code); 
			}
		}
		
		function updateConnectionMode(mode)
		{
			if (mc_toolbar.connection_mc == undefined) {
				return;
			}
			
			mc_toolbar.connection_mc.gotoAndStop(mode);
		}
	}
}
