﻿/** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
**/

package player {
	
	public class PlayerProfiler {

		private static var _instance:PlayerProfiler;
		
		private var _results:Object;
		private var _runningMeasurements:Object;
		private var _enableProfiling:Boolean;

		public function PlayerProfiler(enableProfiling:Boolean) {
			if(_instance != null){
				throw new Error("Singleton")
			}
			_enableProfiling = enableProfiling;
			_results = new Object();
			_runningMeasurements = new Object();
		}
		
		public static function getInstance(enableProfiling:Boolean = false):PlayerProfiler{
			if(_instance == null){
				_instance = new PlayerProfiler(enableProfiling);
			}
			return _instance;
		}
		
		public function start(identifier:String=null):String{
			if(!_enableProfiling){
				return null;
			}
			
			if(identifier == null){
				identifier = _callingMethod();
			}
			
			var uniqueIdentifier:String = identifier + "_" + (new Date()).time;
			
			if(_runningMeasurements[uniqueIdentifier] != null){
				throw new Error("Duplicated measurement id " + uniqueIdentifier);
			}
			
			_runningMeasurements[uniqueIdentifier] = (new Date()).time;
			
			return uniqueIdentifier;
		}
		
		public function end(uniqueIdentifier:String, identifier:String=null):void{
			if(!_enableProfiling){
				return;
			}
			
			if(identifier == null){
				identifier = _callingMethod();
			}
			
			var startTime:Number = _runningMeasurements[uniqueIdentifier];
			
			if(isNaN(startTime)){
				throw new Error("Measurement " + uniqueIdentifier + " not started");
			}
			
			// Results
			var time:Number = (new Date()).time - startTime;
			if(_results[identifier] == null){
				_results[identifier] = {identifier: identifier, count: 1, totalTime: time, averageTime: time};
			} else {
				_results[identifier].count++;
				_results[identifier].totalTime += time;
				_results[identifier].averageTime = _results[identifier].totalTime / _results[identifier].count;
			}
			
			// Clean up
			_runningMeasurements[uniqueIdentifier] = null;
		}

		public function printResults():void{
			if(!_enableProfiling){
				_trace("Profiling is disabled");
			} else {
				_trace(" --- Profiling results ---");
				_trace("");
				
				var sumTotal:Number = 0;
				for each(var resultObj:Object in _results){
					sumTotal += resultObj.totalTime;
					_trace("total: " + resultObj.totalTime + ", avg: " + resultObj.averageTime + ", count: " + resultObj.count + ", id: " + resultObj.identifier);
				}
				_trace("total sum: " + sumTotal);
			}
		}
		
		private function _trace(string:String):void{
			trace("[PROFILER] " + string);
		}
		
		private function _callingMethod():String {
			var e:Error = new Error();
			var line:String = e.getStackTrace().split("at ")[3];
			var startIndex:int = line.indexOf("::");// start of first line
    		var endIndex:int = line.indexOf("()");
			var classAndMethod:Array = line.substring(startIndex + 3, endIndex).split("/");
			return classAndMethod[classAndMethod.length - 1] as String;
		}
	}
}
